@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit).
@@ -23,7 +23,7 @@
          is_running/1, environment/0, rotate_logs/1, force_event_refresh/1,
          start_fhc/0]).
 -export([start/2, stop/1]).
--export([start_apps/1, stop_apps/1]).
+-export([start_apps/1, start_apps/2, stop_apps/1]).
 -export([log_location/1, config_files/0, decrypt_config/2]). %% for testing and mgmt-agent
 
 %%---------------------------------------------------------------------------
@@ -200,6 +200,11 @@
                                    [rabbit_core_metrics_gc]}},
                     {enables,     networking}]}).
 
+-rabbit_boot_step({rabbit_looking_glass,
+                   [{description, "Looking Glass tracer and profiler"},
+                    {mfa,         {rabbit_looking_glass, boot, []}},
+                    {requires,    networking}]}).
+
 %%---------------------------------------------------------------------------
 
 -include("rabbit_framing.hrl").
@@ -212,6 +217,7 @@
 %%----------------------------------------------------------------------------
 
 -type file_suffix() :: binary().
+-type restart_type() :: 'permanent' | 'transient' | 'temporary'.
 %% this really should be an abstract type
 -type log_location() :: 'tty' | 'undefined' | file:filename().
 -type param() :: atom().
@@ -248,6 +254,7 @@
 -spec boot_delegate() -> 'ok'.
 -spec recover() -> 'ok'.
 -spec start_apps([app_name()]) -> 'ok'.
+-spec start_apps([app_name()], [{app_name(), restart_type()}]) -> 'ok'.
 -spec stop_apps([app_name()]) -> 'ok'.
 
 %%----------------------------------------------------------------------------
@@ -436,7 +443,7 @@ stop() ->
         undefined -> ok;
         _         ->
             rabbit_log:info("RabbitMQ hasn't finished starting yet. Waiting for startup to finish before stopping..."),
-            await_startup(true)
+            wait_for_boot_to_finish()
     end,
     rabbit_log:info("RabbitMQ is asked to stop...~n", []),
     Apps = ?APPS ++ rabbit_plugins:active(),
@@ -447,22 +454,31 @@ stop_and_halt() ->
     try
         stop()
     catch Type:Reason ->
-        rabbit_log:error("Error trying to stop RabbitMQ: ~p:~p", [Type, Reason])
+        rabbit_log:error("Error trying to stop RabbitMQ: ~p:~p", [Type, Reason]),
+        erlang:error({Type, Reason})
     after
-        AppsLeft = [ A || {A, _, _} <- application:which_applications() ],
-        rabbit_log:info(
-            lists:flatten(["Halting Erlang VM with the following applications:~n",
-                           ["    ~p~n" || _ <- AppsLeft]]),
-            AppsLeft),
-        %% Also duplicate this information to stderr, so console where
-        %% foreground broker was running (or systemd journal) will
-        %% contain information about graceful termination.
-        io:format(standard_error, "Gracefully halting Erlang VM~n", []),
-        init:stop()
+        %% Enclose all the logging in the try block.
+        %% init:stop() will be called regardless of any errors.
+        try
+            AppsLeft = [ A || {A, _, _} <- application:which_applications() ],
+            rabbit_log:info(
+                lists:flatten(["Halting Erlang VM with the following applications:~n",
+                               ["    ~p~n" || _ <- AppsLeft]]),
+                AppsLeft),
+            %% Also duplicate this information to stderr, so console where
+            %% foreground broker was running (or systemd journal) will
+            %% contain information about graceful termination.
+            io:format(standard_error, "Gracefully halting Erlang VM~n", [])
+        after
+            init:stop()
+        end
     end,
     ok.
 
 start_apps(Apps) ->
+    start_apps(Apps, []).
+
+start_apps(Apps, RestartTypes) ->
     app_utils:load_applications(Apps),
 
     ConfigEntryDecoder = case application:get_env(rabbit, config_entry_decoder) of
@@ -502,7 +518,8 @@ start_apps(Apps) ->
         true  -> ok                    %% will run during start of rabbit app
     end,
     ok = app_utils:start_applications(OrderedApps,
-                                      handle_app_error(could_not_start)).
+                                      handle_app_error(could_not_start),
+                                      RestartTypes).
 
 %% This function retrieves the correct IoDevice for requesting
 %% input. The problem with using the default IoDevice is that
@@ -576,9 +593,11 @@ decrypt_list([{Key, Value}|Tail], Algo, Acc) when Key =/= encrypted ->
 decrypt_list([Value|Tail], Algo, Acc) ->
     decrypt_list(Tail, Algo, [decrypt(Value, Algo)|Acc]).
 
+stop_apps([]) ->
+    ok;
 stop_apps(Apps) ->
     rabbit_log:info(
-        lists:flatten(["Stopping RabbitMQ applications and their dependencies in the following order: ~n",
+        lists:flatten(["Stopping RabbitMQ applications and their dependencies in the following order:~n",
                        ["    ~p~n" || _ <- Apps]]),
         lists:reverse(Apps)),
     ok = app_utils:stop_applications(
@@ -598,19 +617,32 @@ handle_app_error(Term) ->
     end.
 
 await_startup() ->
-    await_startup(false).
+    case is_booting() of
+        true -> wait_for_boot_to_finish();
+        false ->
+            case is_running() of
+                true -> ok;
+                false -> wait_for_boot_to_start(),
+                         wait_for_boot_to_finish()
+            end
+    end.
 
-await_startup(HaveSeenRabbitBoot) ->
-    %% We don't take absence of rabbit_boot as evidence we've started,
-    %% since there's a small window before it is registered.
+is_booting() ->
+    whereis(rabbit_boot) /= undefined.
+
+wait_for_boot_to_start() ->
     case whereis(rabbit_boot) of
-        undefined -> case HaveSeenRabbitBoot orelse is_running() of
-                         true  -> ok;
-                         false -> timer:sleep(100),
-                                  await_startup(false)
-                     end;
+        undefined -> timer:sleep(100),
+                     wait_for_boot_to_start();
+        _         -> ok
+    end.
+
+wait_for_boot_to_finish() ->
+    case whereis(rabbit_boot) of
+        undefined -> true = is_running(),
+                     ok;
         _         -> timer:sleep(100),
-                     await_startup(true)
+                     wait_for_boot_to_finish()
     end.
 
 status() ->
@@ -621,7 +653,8 @@ status() ->
           {erlang_version,       erlang:system_info(system_version)},
           {memory,               rabbit_vm:memory()},
           {alarms,               alarms()},
-          {listeners,            listeners()}],
+          {listeners,            listeners()},
+          {vm_memory_calculation_strategy, vm_memory_monitor:get_memory_calculation_strategy()}],
     S2 = rabbit_misc:filter_exit_map(
            fun ({Key, {M, F, A}}) -> {Key, erlang:apply(M, F, A)} end,
            [{vm_memory_high_watermark, {vm_memory_monitor,
