@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ Federation.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_federation_link_util).
@@ -300,6 +300,8 @@ disposable_connection_call(Params, Method, ErrFun) ->
                 amqp_channel:call(Ch, Method)
             catch exit:{{shutdown, {connection_closing,
                                     {server_initiated_close, Code, Txt}}}, _} ->
+                    ErrFun(Code, Txt);
+                    exit:{{shutdown, {server_initiated_close, Code, Txt}}, _} ->
                     ErrFun(Code, Txt)
             after
                 ensure_connection_closed(Conn)
