@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is GoPivotal, Inc.
-%%   Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%%   Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_util).
@@ -28,17 +28,23 @@
 -export([with_decode/4, not_found/3, amqp_request/4]).
 -export([with_channel/4, with_channel/5]).
 -export([props_to_method/2, props_to_method/4]).
--export([all_or_one_vhost/2, http_to_amqp/5, reply/3, filter_vhost/3]).
+-export([all_or_one_vhost/2, http_to_amqp/5, reply/3, responder_map/1,
+         filter_vhost/3]).
 -export([filter_conn_ch_list/3, filter_user/2, list_login_vhosts/2]).
 -export([with_decode/5, decode/1, decode/2, set_resp_header/3,
          args/1]).
 -export([reply_list/3, reply_list/5, reply_list/4,
-         sort_list/2, destination_type/1, reply_list_or_paginate/3]).
+         sort_list/2, destination_type/1, reply_list_or_paginate/3
+         ]).
 -export([post_respond/1, columns/1, is_monitor/1]).
 -export([list_visible_vhosts/1, b64decode_or_throw/1, no_range/0, range/1,
          range_ceil/1, floor/2, ceil/1, ceil/2]).
--export([pagination_params/1, maybe_filter_by_keyword/4,
-         get_value_param/2]).
+-export([pagination_params/1,
+         maybe_filter_by_keyword/4,
+         get_value_param/2,
+         augment_resources/6
+        ]).
+-export([get_path_prefix/0]).
 
 -import(rabbit_misc, [pget/2]).
 
@@ -49,10 +55,17 @@
 -define(FRAMING, rabbit_framing_amqp_0_9_1).
 -define(DEFAULT_PAGE_SIZE, 100).
 -define(MAX_PAGE_SIZE, 500).
+
+-define(MAX_RANGE, 500).
+
 -record(pagination, {page = undefined, page_size = undefined,
                      name = undefined, use_regex = undefined}).
 
--define(MAX_RANGE, 500).
+-record(aug_ctx, {req_data :: cowboy_req:req(),
+                  pagination :: #pagination{},
+                  sort = [] :: [atom()],
+                  columns = [] :: [atom()],
+                  data :: term()}).
 
 %%--------------------------------------------------------------------
 
@@ -193,14 +206,32 @@ destination_type(ReqData) ->
         <<"q">> -> queue
     end.
 
+%% Provides a map of content type-to-responder that are supported by
+%% reply/3. The map can be used in the content_types_provided/2 callback
+%% used by cowboy_rest. Responder functions must be
+%% exported from the caller module and must use reply/3
+%% under the hood.
+responder_map(FunctionName) ->
+    [
+      {<<"application/json">>, FunctionName},
+      {<<"application/bert">>, FunctionName}
+    ].
+
+reply({halt, _, _} = Reply, _ReqData, _Context) ->
+    Reply;
 reply(Facts, ReqData, Context) ->
     reply0(extract_columns(Facts, ReqData), ReqData, Context).
 
 reply0(Facts, ReqData, Context) ->
-    ReqData1 = set_resp_header(<<"Cache-Control">>, "no-cache", ReqData),
+    ReqData1 = set_resp_header(<<"cache-control">>, "no-cache", ReqData),
     try
-        {mochijson2:encode(rabbit_mgmt_format:format_nulls(Facts)), ReqData1,
-         Context}
+        case cowboy_req:meta(media_type, ReqData1) of
+            {{<<"application">>, <<"bert">>, _}, _} ->
+                {term_to_binary(Facts), ReqData1, Context};
+            _ ->
+                {mochijson2:encode(rabbit_mgmt_format:format_nulls(Facts)),
+                 ReqData1, Context}
+        end
     catch exit:{json_encode, E} ->
             Error = iolist_to_binary(
                       io_lib:format("JSON encode error: ~p", [E])),
@@ -223,7 +254,7 @@ get_value_param(Name, ReqData) ->
 
 reply_list(Facts, DefaultSorts, ReqData, Context, Pagination) ->
     SortList =
-    sort_list(
+    sort_list_and_paginate(
           extract_columns_list(Facts, ReqData),
           DefaultSorts,
           get_value_param(<<"sort">>, ReqData),
@@ -238,10 +269,18 @@ get_sort_reverse(ReqData) ->
         V -> list_to_atom(V)
     end.
 
-reply_list_or_paginate(Facts, ReqData, Context) ->
+
+-spec is_pagination_requested(#pagination{} | undefined) -> boolean().
+is_pagination_requested(undefined) ->
+    false;
+is_pagination_requested(#pagination{}) ->
+    true.
+
+
+with_valid_pagination(ReqData, Context, Fun) ->
     try
         Pagination = pagination_params(ReqData),
-        reply_list(Facts, ["vhost", "name"], ReqData, Context, Pagination)
+        Fun(Pagination)
     catch error:badarg ->
         Reason = iolist_to_binary(
                io_lib:format("Pagination parameters are invalid", [])),
@@ -251,27 +290,159 @@ reply_list_or_paginate(Facts, ReqData, Context) ->
             invalid_pagination(ErrorType, Reason, ReqData, Context)
     end.
 
+reply_list_or_paginate(Facts, ReqData, Context) ->
+    with_valid_pagination(
+      ReqData, Context,
+      fun(Pagination) ->
+              reply_list(Facts, ["vhost", "name"], ReqData, Context, Pagination)
+      end).
 
-sort_list(Facts, Sorts) -> sort_list(Facts, Sorts, undefined, false,
+merge_sorts(DefaultSorts, Extra) ->
+    case Extra of
+        undefined -> DefaultSorts;
+        Extra     -> [Extra | DefaultSorts]
+    end.
+
+%% Resource augmentation. Works out the most optimal configuration of the operations:
+%% sort, page, augment and executes it returning the result.
+
+column_strategy(all, _) -> extended;
+column_strategy(Cols, BasicColumns) ->
+    case Cols -- BasicColumns of
+        [] -> basic;
+        _ -> extended
+    end.
+
+% columns are [[binary()]] - this takes the first item
+columns_as_strings(all) -> all;
+columns_as_strings(Columns0) ->
+    [rabbit_data_coercion:to_list(C) || [C | _] <- Columns0].
+
+augment_resources(Resources, DefaultSort, BasicColumns, ReqData, Context,
+                  AugmentFun) ->
+    with_valid_pagination(ReqData, Context,
+                          fun (Pagination) ->
+                                  augment_resources0(Resources, DefaultSort,
+                                                     BasicColumns, Pagination,
+                                                     ReqData, AugmentFun)
+                          end).
+
+augment_resources0(Resources, DefaultSort, BasicColumns, Pagination, ReqData,
+                   AugmentFun) ->
+    SortFun = fun (AugCtx) -> sort(DefaultSort, AugCtx) end,
+    AugFun = fun (AugCtx) -> augment(AugmentFun, AugCtx) end,
+    PageFun = fun page/1,
+    Pagination = pagination_params(ReqData),
+    Sort = def(get_value_param(<<"sort">>, ReqData), DefaultSort),
+    Columns = def(columns(ReqData), all),
+    ColumnsAsStrings = columns_as_strings(Columns),
+    Pipeline =
+        case {Pagination =/= undefined,
+              column_strategy(Sort, BasicColumns),
+              column_strategy(ColumnsAsStrings, BasicColumns)} of
+            {false, basic, basic} -> % no pagination, no extended fields
+                [SortFun];
+            {false, _, _} ->
+                % no pagination, extended columns means we need to augment all - SLOW
+                [AugFun, SortFun];
+            {true, basic, basic} ->
+                [SortFun, PageFun];
+            {true, extended, _} ->
+                % pagination with extended sort columns - SLOW
+                [AugFun, SortFun, PageFun];
+            {true, basic, extended} ->
+                % pagination with extended columns and sorting on basic
+                % here we can reduce the augementation set before
+                % augmenting
+                [SortFun, PageFun, AugFun]
+        end,
+    #aug_ctx{data = {_, Reply}} = run_augmentation(
+                                    #aug_ctx{req_data = ReqData,
+                                             pagination = Pagination,
+                                             sort = Sort,
+                                             columns = Columns,
+                                             data = {loaded, Resources}},
+                                    Pipeline),
+    rabbit_mgmt_format:strip_pids(Reply).
+
+run_augmentation(C, []) -> C;
+run_augmentation(C, [Next | Rem]) ->
+    C2 = Next(C),
+    run_augmentation(C2, Rem).
+
+sort(DefaultSort, Ctx = #aug_ctx{data = Data0,
+                                 req_data = ReqData,
+                                 sort = Sort}) ->
+    Data1 = get_data(Data0),
+    Data = sort_list(Data1, DefaultSort, Sort,
+                     get_sort_reverse(ReqData)),
+    Ctx#aug_ctx{data = update_data(Data0, {sorted, Data})}.
+
+page(Ctx = #aug_ctx{data = Data0,
+                    pagination = Pagination}) ->
+    Data = filter_and_paginate(get_data(Data0), Pagination),
+    Ctx#aug_ctx{data = update_data(Data0, {paged, Data})}.
+
+update_data({paged, Old}, New) ->
+    {paged, rabbit_misc:pset(items, get_data(New), Old)};
+update_data(_, New) ->
+    New.
+
+augment(AugmentFun, Ctx = #aug_ctx{data = Data0, req_data = ReqData}) ->
+    Data1 = get_data(Data0),
+    Data = AugmentFun(Data1, ReqData),
+    Ctx#aug_ctx{data = update_data(Data0, {augmented, Data})}.
+
+get_data({paged, Data}) ->
+    rabbit_misc:pget(items, Data);
+get_data({_, Data}) ->
+    Data.
+
+get_path_prefix() ->
+    EnvPrefix = rabbit_misc:get_env(rabbitmq_management, path_prefix, ""),
+    fixup_prefix(EnvPrefix).
+
+fixup_prefix("") ->
+    "";
+fixup_prefix([Char|_Rest]=EnvPrefix) when is_list(EnvPrefix), Char =:= $/ ->
+    EnvPrefix;
+fixup_prefix(EnvPrefix) when is_list(EnvPrefix) ->
+    "/" ++ EnvPrefix;
+fixup_prefix(EnvPrefix) when is_binary(EnvPrefix) ->
+    fixup_prefix(rabbit_data_coercion:to_list(EnvPrefix)).
+
+%% XXX sort_list_and_paginate/2 is a more proper name for this function, keeping it
+%% with this name for backwards compatibility
+-spec sort_list([Fact], [string()]) -> [Fact] when
+      Fact :: [{atom(), term()}].
+sort_list(Facts, Sorts) -> sort_list_and_paginate(Facts, Sorts, undefined, false,
   undefined).
 
-sort_list(Facts, DefaultSorts, Sort, Reverse, Pagination) ->
-    SortList = case Sort of
-           undefined -> DefaultSorts;
-           Extra     -> [Extra | DefaultSorts]
-           end,
+-spec sort_list([Fact], [SortColumn], [SortColumn] | undefined, boolean()) -> [Fact] when
+      Fact :: [{atom(), term()}],
+      SortColumn :: string().
+sort_list(Facts, _, [], _) ->
+    %% Do not sort when we are explicitly requsted to sort with an
+    %% empty sort columns list. Note that this clause won't match when
+    %% 'sort' parameter is not provided in a HTTP request at all.
+    Facts;
+sort_list(Facts, DefaultSorts, Sort, Reverse) ->
+    SortList = merge_sorts(DefaultSorts, Sort),
     %% lists:sort/2 is much more expensive than lists:sort/1
     Sorted = [V || {_K, V} <- lists:sort(
                                 [{sort_key(F, SortList), F} || F <- Facts])],
+    maybe_reverse(Sorted, Reverse).
 
+sort_list_and_paginate(Facts, DefaultSorts, Sort, Reverse, Pagination) ->
+    filter_and_paginate(sort_list(Facts, DefaultSorts, Sort, Reverse), Pagination).
+
+filter_and_paginate(Sorted, Pagination) ->
     ContextList = maybe_filter_context(Sorted, Pagination),
-    range_filter(maybe_reverse(ContextList, Reverse), Pagination, Sorted).
+    range_filter(ContextList, Pagination, Sorted).
 
 %%
 %% Filtering functions
 %%
-
-
 maybe_filter_context(List, #pagination{name = Name, use_regex = UseRegex}) when
       is_list(Name) ->
     lists:filter(fun(ListF) ->
@@ -406,9 +577,19 @@ pget_bin(Key, List, Default) ->
         {[{_K, V}], _} -> V;
         {[],        _} -> Default
     end.
+maybe_pagination(Item, false, ReqData) ->
+    extract_column_items(Item, columns(ReqData));
+maybe_pagination([{items, Item} | T], true, ReqData) ->
+    [{items, extract_column_items(Item, columns(ReqData))} |
+     maybe_pagination(T, true, ReqData)];
+maybe_pagination([H | T], true, ReqData) ->
+    [H | maybe_pagination(T,true, ReqData)];
+maybe_pagination(Item, true, ReqData) ->
+    [maybe_pagination(X, true, ReqData) || X <- Item].
 
 extract_columns(Item, ReqData) ->
-    extract_column_items(Item, columns(ReqData)).
+    maybe_pagination(Item, is_pagination_requested(pagination_params(ReqData)),
+		     ReqData).
 
 extract_columns_list(Items, ReqData) ->
     Cols = columns(ReqData),
@@ -492,7 +673,7 @@ with_decode(Keys, Body, ReqData, Context, Fun) ->
     case decode(Keys, Body) of
         {error, Reason}    -> bad_request(Reason, ReqData, Context);
         {ok, Values, JSON} -> try
-                                  Fun(Values, JSON)
+                                  Fun(Values, JSON, ReqData)
                               catch {error, Error} ->
                                       bad_request(Error, ReqData, Context)
                               end
@@ -546,7 +727,8 @@ http_to_amqp(MethodName, ReqData, Context, Transformers, Extra) ->
                             bad_request(Error, ReqData1, Context)
                     end;
                 {error, Reason} ->
-                    bad_request(Reason, ReqData1, Context)
+                    bad_request(rabbit_mgmt_format:escape_html_tags(Reason),
+                                ReqData1, Context)
             end
     end.
 
@@ -634,6 +816,8 @@ with_channel(VHost, ReqData,
             end;
         {error, {auth_failure, Msg}} ->
             not_authorised(Msg, ReqData, Context);
+        {error, not_allowed} ->
+            not_authorised(<<"Access refused.">>, ReqData, Context);
         {error, access_refused} ->
             not_authorised(<<"Access refused.">>, ReqData, Context);
         {error, {nodedown, N}} ->
@@ -648,7 +832,7 @@ bad_request_exception(Code, Reason, ReqData, Context) ->
                 ReqData, Context).
 
 all_or_one_vhost(ReqData, Fun) ->
-    case rabbit_mgmt_util:vhost(ReqData) of
+    case vhost(ReqData) of
         none      -> lists:append([Fun(V) || V <- rabbit_vhost:list()]);
         not_found -> vhost_not_found;
         VHost     -> Fun(VHost)
@@ -689,9 +873,7 @@ post_respond({true, ReqData, Context}) ->
 post_respond({halt, ReqData, Context}) ->
     {halt, ReqData, Context};
 post_respond({JSON, ReqData, Context}) ->
-    {true, set_resp_header(
-             <<"Content-Type">>, "application/json",
-             cowboy_req:set_resp_body(JSON, ReqData)), Context}.
+    {true, cowboy_req:set_resp_body(JSON, ReqData), Context}.
 
 is_admin(T)       -> intersects(T, [administrator]).
 is_policymaker(T) -> intersects(T, [administrator, policymaker]).
@@ -806,3 +988,5 @@ int(Name, ReqData) ->
                      end
     end.
 
+def(undefined, Def) -> Def;
+def(V, _) -> V.
