@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is GoPivotal, Inc.
-%%   Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%%   Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 
 -module(rabbit_mgmt_wm_consumers).
 
@@ -35,7 +35,7 @@ variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
 
 content_types_provided(ReqData, Context) ->
-   {[{<<"application/json">>, to_json}], ReqData, Context}.
+   {rabbit_mgmt_util:responder_map(to_json), ReqData, Context}.
 
 resource_exists(ReqData, Context) ->
     {case rabbit_mgmt_util:vhost(ReqData) of
@@ -50,10 +50,10 @@ to_json(ReqData, Context = #context{user = User}) ->
               VHost -> VHost
           end,
 
-    Consumers = rabbit_mgmt_format:strip_pids(
-                          rabbit_mgmt_db:get_all_consumers(Arg)),
+    Consumers = rabbit_mgmt_format:strip_pids(rabbit_mgmt_db:get_all_consumers(Arg)),
+    Formatted = [rabbit_mgmt_format:format_consumer_arguments(C) || C <- Consumers],
     rabbit_mgmt_util:reply_list(
-      filter_user(Consumers, User), ReqData, Context).
+      filter_user(Formatted, User), ReqData, Context).
 
 is_authorized(ReqData, Context) ->
     rabbit_mgmt_util:is_authorized(ReqData, Context).
