@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is GoPivotal, Inc.
-%%   Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%%   Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_format).
@@ -21,13 +21,14 @@
 -export([protocol/1, resource/1, queue/1, queue_state/1]).
 -export([exchange/1, user/1, internal_user/1, binding/1, url/2]).
 -export([pack_binding_props/2, tokenise/1]).
--export([to_amqp_table/1, listener/1, properties/1, basic_properties/1]).
+-export([to_amqp_table/1, listener/1, web_context/1, properties/1, basic_properties/1]).
 -export([record/2, to_basic_properties/1]).
 -export([addr/1, port/1]).
--export([format_nulls/1]).
+-export([format_nulls/1, escape_html_tags/1]).
 -export([print/2, print/1]).
 
 -export([format_queue_stats/1, format_channel_stats/1,
+         format_consumer_arguments/1,
          format_arguments/1, format_connection_created/1,
          format_accept_content/1, format_args/1]).
 
@@ -35,7 +36,7 @@
 
 -export([clean_consumer_details/1, clean_channel_details/1]).
 
--import(rabbit_misc, [pget/2, pset/3]).
+-import(rabbit_misc, [pget/2, pget/3, pset/3]).
 
 -include_lib("rabbit_common/include/rabbit.hrl").
 -include_lib("rabbit_common/include/rabbit_framing.hrl").
@@ -75,10 +76,10 @@ format_queue_stats({disk_writes, _}) ->
 format_queue_stats(Stat) ->
     [Stat].
 
-format_channel_stats({idle_since, Value}) ->
-    {idle_since, now_to_str(Value)};
-format_channel_stats(Stat) ->
-    Stat.
+format_channel_stats([{idle_since, Value} | Rest]) ->
+    [{idle_since, now_to_str(Value)} | Rest];
+format_channel_stats(Stats) ->
+    Stats.
 
 format_arguments({arguments, Value}) ->
     {arguments, amqp_table(Value)};
@@ -253,28 +254,59 @@ listener(#listener{node = Node, protocol = Protocol,
      {protocol, Protocol},
      {ip_address, ip(IPAddress)},
      {port, Port},
-     {socket_opts, opts(Opts)}].
+     {socket_opts, format_socket_opts(Opts)}].
 
-opts(Opts) ->
-    opts(Opts, []).
+web_context(Props0) ->
+    SslOpts = pget(ssl_opts, Props0, []),
+    Props   = proplists:delete(ssl_opts, Props0),
+    [{ssl_opts, format_socket_opts(SslOpts)} | Props].
 
-opts([], Acc) ->
+format_socket_opts(Opts) ->
+    format_socket_opts(Opts, []).
+
+format_socket_opts([], Acc) ->
     lists:reverse(Acc);
+%% for HTTP API listeners this will be included into
+%% socket_opts
+format_socket_opts([{ssl_opts, Value} | Tail], Acc) ->
+    format_socket_opts(Tail, [{ssl_opts, format_socket_opts(Value)} | Acc]);
+%% exclude options that have values that are nested
+%% data structures or may include functions. They are fairly
+%% obscure and not worth reporting via HTTP API.
+format_socket_opts([{verify_fun, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+format_socket_opts([{crl_cache, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+format_socket_opts([{partial_chain, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+format_socket_opts([{user_lookup_fun, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+format_socket_opts([{sni_fun, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+format_socket_opts([{reuse_session, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+%% we do not want to report configured cipher suites, even
+%% though formatting them is straightforward
+format_socket_opts([{ciphers, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+%% single atom options, e.g. `binary`
+format_socket_opts([Head | Tail], Acc) when is_atom(Head) ->
+    format_socket_opts(Tail, [{Head, true} | Acc]);
 %% verify_fun value is a tuple that includes a function
-opts([_Head = {verify_fun, _Value} | Tail], Acc) ->
-    opts(Tail, Acc);
-opts([Head = {Name, Value} | Tail], Acc) when is_list(Value) ->
+format_socket_opts([_Head = {verify_fun, _Value} | Tail], Acc) ->
+    format_socket_opts(Tail, Acc);
+format_socket_opts([Head = {Name, Value} | Tail], Acc) when is_list(Value) ->
     case io_lib:printable_unicode_list(Value) of
-        true -> opts(Tail, [{Name, unicode:characters_to_binary(Value)} | Acc]);
-        false -> opts(Tail, [Head | Acc])
+        true -> format_socket_opts(Tail, [{Name, unicode:characters_to_binary(Value)} | Acc]);
+        false -> format_socket_opts(Tail, [Head | Acc])
     end;
-opts([{Name, Value} | Tail], Acc) when is_tuple(Value) ->
-    opts(Tail, [{Name, tuple_to_list(Value)} | Acc]);
+format_socket_opts([{Name, Value} | Tail], Acc) when is_tuple(Value) ->
+    format_socket_opts(Tail, [{Name, tuple_to_list(Value)} | Acc]);
 %% exclude functions from JSON encoding
-opts([_Head = {_Name, Value} | Tail], Acc) when is_function(Value) ->
-    opts(Tail, Acc);
-opts([Head | Tail], Acc) ->
-    opts(Tail, [Head | Acc]).
+format_socket_opts([_Head = {_Name, Value} | Tail], Acc) when is_function(Value) ->
+    format_socket_opts(Tail, Acc);
+format_socket_opts([Head | Tail], Acc) ->
+    format_socket_opts(Tail, [Head | Acc]).
 
 pack_binding_props(<<"">>, []) ->
     <<"~">>;
@@ -490,12 +522,33 @@ format_null_item([{_K, _V} | _T] = L) ->
 format_null_item(Value) ->
     Value.
 
+
+-spec escape_html_tags(string()) -> binary().
+
+escape_html_tags(S) ->
+    escape_html_tags(rabbit_data_coercion:to_list(S), []).
+
+
+-spec escape_html_tags(string(), string()) -> binary().
+
+escape_html_tags([], Acc) ->
+    rabbit_data_coercion:to_binary(lists:reverse(Acc));
+escape_html_tags("<" ++ Rest, Acc) ->
+    escape_html_tags(Rest, lists:reverse("&lt;", Acc));
+escape_html_tags(">" ++ Rest, Acc) ->
+    escape_html_tags(Rest, lists:reverse("&gt;", Acc));
+escape_html_tags("&" ++ Rest, Acc) ->
+    escape_html_tags(Rest, lists:reverse("&amp;", Acc));
+escape_html_tags([C | Rest], Acc) ->
+    escape_html_tags(Rest, [C | Acc]).
+
+
 -spec clean_consumer_details(proplists:proplist()) -> proplists:proplist().
 clean_consumer_details(Obj) ->
      case pget(consumer_details, Obj) of
          undefined -> Obj;
          Cds ->
-             Cons = [clean_channel_details(Con) || Con <- Cds],
+             Cons = [format_consumer_arguments(clean_channel_details(Con)) || Con <- Cds],
              pset(consumer_details, Cons, Obj)
      end.
 
@@ -510,6 +563,16 @@ clean_channel_details(Obj) ->
                   Obj0)
      end.
 
+-spec format_consumer_arguments(proplists:proplist()) -> proplists:proplist().
+format_consumer_arguments(Obj) ->
+    case pget(arguments, Obj) of
+         undefined -> Obj;
+         []        -> pset(arguments, {struct, []}, Obj);
+         Args      -> pset(arguments, amqp_table(Args), Obj)
+     end.
+
+%% Converts a proplist into an AMQP 0-9-1 table.
+%% For JSON serialisation formatting see amqp_table/1.
 args({struct, L}) -> args(L);
 args(L)           -> to_amqp_table(L).
 
