@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 %%
 -module(rabbit_mgmt_metrics_collector).
 
@@ -20,6 +20,7 @@
 -include("rabbit_mgmt_metrics.hrl").
 
 -behaviour(gen_server).
+-compile({no_auto_import, [ceil/1]}).
 
 -spec start_link(atom()) -> rabbit_types:ok_pid_or_error().
 
@@ -31,7 +32,6 @@
 -export([index_table/2]).
 -export([reset_all/0]).
 
--import(rabbit_misc, [pget/3]).
 -import(rabbit_mgmt_data, [lookup_element/3]).
 
 -record(state, {table, interval, policies, rates_mode, lookup_queue,
@@ -70,7 +70,7 @@ reset_lookups(Table) ->
 init([Table]) ->
     {RatesMode, Policies} = load_config(),
     Policy = retention_policy(Table),
-    Interval = take_smaller(proplists:get_value(Policy, Policies)) * 1000,
+    Interval = take_smaller(proplists:get_value(Policy, Policies, [])) * 1000,
     erlang:send_after(Interval, self(), collect_metrics),
     {ok, #state{table = Table, interval = Interval,
                 policies = {proplists:get_value(basic, Policies),
@@ -132,24 +132,37 @@ retention_policy(node_metrics) -> basic;
 retention_policy(node_node_metrics) -> global.
 
 take_smaller(Policies) ->
-    lists:min([I || {_, I} <- Policies]).
+    Intervals = [I || {_, I} <- Policies],
+    case Intervals of
+        [] -> throw(missing_sample_retention_policies);
+        _ -> lists:min(Intervals)
+    end.
 
 insert_old_aggr_stats(NextStats, Id, Stat) ->
     dict:store(Id, Stat, NextStats).
 
-handle_deleted_queues(queue_coarse_metrics, Remainders, GPolicies) ->
+handle_deleted_queues(queue_coarse_metrics, Remainders,
+                      #state{policies = {BPolicies, _, GPolicies}}) ->
     TS = exometer_slide:timestamp(),
     lists:foreach(fun ({Queue, {R, U, M}}) ->
                             NegStats = ?vhost_msg_stats(-R, -U, -M),
                             [insert_entry(vhost_msg_stats, vhost(Queue), TS,
                                           NegStats, Size, Interval, true)
-                             || {Size, Interval} <- GPolicies]
+                             || {Size, Interval} <- GPolicies],
+                            % zero out msg stats to avoid duplicating msg
+                            % stats when a master queue is migrated
+                            QNegStats = ?queue_msg_stats(0, 0, 0),
+                            [insert_entry(queue_msg_stats, Queue, TS,
+                                          QNegStats, Size, Interval, false)
+                             || {Size, Interval} <- BPolicies],
+                            ets:delete(queue_stats, Queue),
+                            ets:delete(queue_process_stats, Queue)
                   end,
                   dict:to_list(Remainders));
 handle_deleted_queues(_T, _R, _P) -> ok.
 
 aggregate_metrics(Timestamp, #state{table = Table,
-                                    policies = {_, _, GPolicies}} = State0) ->
+                                    policies = {_, _, _GPolicies}} = State0) ->
     Table = State0#state.table,
     {Next, Ops, #state{old_aggr_stats = Remainders}} =
         ets:foldl(fun(R, {NextStats, O, State}) ->
@@ -160,7 +173,7 @@ aggregate_metrics(Timestamp, #state{table = Table,
                       Acc
               end, no_acc, Ops),
 
-    handle_deleted_queues(Table, Remainders, GPolicies),
+    handle_deleted_queues(Table, Remainders, State0),
     State0#state{old_aggr_stats = Next}.
 
 exec_table_ops(Table, Timestamp, TableOps) ->
@@ -180,12 +193,17 @@ exec_table_ops(Table, Timestamp, TableOps) ->
 
 aggregate_entry({Id, Metrics}, NextStats, Ops0,
                 #state{table = connection_created} = State) ->
-    Ftd = rabbit_mgmt_format:format(
-        Metrics,
-        {fun rabbit_mgmt_format:format_connection_created/1, true}),
-    Entry = ?connection_created_stats(Id, pget(name, Ftd, unknown), Ftd),
-    Ops = insert_op(connection_created_stats, Id, Entry, Ops0),
-    {NextStats, Ops, State};
+    case ets:lookup(connection_created_stats, Id) of
+        [] ->
+            Ftd = rabbit_mgmt_format:format(
+                    Metrics,
+                    {fun rabbit_mgmt_format:format_connection_created/1, true}),
+            Entry = ?connection_created_stats(Id, pget(name, Ftd, unknown), Ftd),
+            Ops = insert_op(connection_created_stats, Id, Entry, Ops0),
+            {NextStats, Ops, State};
+        _ ->
+            {NextStats, Ops0, State}
+    end;
 aggregate_entry({Id, Metrics}, NextStats, Ops0,
                 #state{table = connection_metrics} = State) ->
     Entry = ?connection_stats(Id, Metrics),
@@ -217,15 +235,22 @@ aggregate_entry({Id, RecvOct, SendOct, _Reductions, 1}, NextStats, Ops0,
     {NextStats, Ops1, State};
 aggregate_entry({Id, Metrics}, NextStats, Ops0,
                 #state{table = channel_created} = State) ->
-    Ftd = rabbit_mgmt_format:format(Metrics, {[], false}),
-    Entry = ?channel_created_stats(Id, pget(name, Ftd, unknown), Ftd),
-    Ops = insert_op(channel_created_stats, Id, Entry, Ops0),
-    {NextStats, Ops, State};
+    case ets:lookup(channel_created_stats, Id) of
+        [] ->
+            Ftd = rabbit_mgmt_format:format(Metrics, {[], false}),
+            Entry = ?channel_created_stats(Id, pget(name, Ftd, unknown), Ftd),
+            Ops = insert_op(channel_created_stats, Id, Entry, Ops0),
+            {NextStats, Ops, State};
+        _ ->
+            {NextStats, Ops0, State}
+    end;
 aggregate_entry({Id, Metrics}, NextStats, Ops0,
                 #state{table = channel_metrics} = State) ->
-    Ftd = rabbit_mgmt_format:format(Metrics,
-                    {fun rabbit_mgmt_format:format_channel_stats/1, true}),
-
+    %% First metric must be `idle_since` (if available), as expected by
+    %% `rabbit_mgmt_format:format_channel_stats`. This is a performance
+    %% optimisation that avoids traversing the whole list when only
+    %% one element has to be formatted.
+    Ftd = rabbit_mgmt_format:format_channel_stats(Metrics),
     Entry = ?channel_stats(Id, Ftd),
     Ops = insert_op(channel_stats, Id, Entry, Ops0),
     {NextStats, Ops, State};
@@ -379,13 +404,18 @@ aggregate_entry({Id, Reductions}, NextStats, Ops0,
 aggregate_entry({Id, Exclusive, AckRequired, PrefetchCount, Args},
                 NextStats, Ops0,
                 #state{table = consumer_created} = State) ->
-    Fmt = rabbit_mgmt_format:format([{exclusive, Exclusive},
-                                     {ack_required, AckRequired},
-                                     {prefetch_count, PrefetchCount},
-                                     {arguments, Args}], {[], false}),
-    Entry = ?consumer_stats(Id, Fmt),
-    Ops = insert_with_index_op(consumer_stats, Id, Entry, Ops0),
-    {NextStats, Ops ,State};
+    case ets:lookup(consumer_stats, Id) of
+        [] ->
+            Fmt = rabbit_mgmt_format:format([{exclusive, Exclusive},
+                                             {ack_required, AckRequired},
+                                             {prefetch_count, PrefetchCount},
+                                             {arguments, Args}], {[], false}),
+            Entry = ?consumer_stats(Id, Fmt),
+            Ops = insert_with_index_op(consumer_stats, Id, Entry, Ops0),
+            {NextStats, Ops ,State};
+        _ ->
+            {NextStats, Ops0, State}
+    end;
 aggregate_entry({Id, Metrics, 0}, NextStats, Ops0,
                 #state{table = queue_metrics,
                        policies = {BPolicies, _, GPolicies},
@@ -618,7 +648,7 @@ index_table(node_node_coarse_stats, node) -> node_node_coarse_stats_node_index.
 
 load_config() ->
     RatesMode = rabbit_mgmt_agent_config:get_env(rates_mode),
-    Policies = rabbit_mgmt_agent_config:get_env(sample_retention_policies),
+    Policies = rabbit_mgmt_agent_config:get_env(sample_retention_policies, []),
     {RatesMode, Policies}.
 
 ceil(X) when X < 0 ->
@@ -631,3 +661,13 @@ ceil(X) ->
     end.
 
 pget(Key, List) -> pget(Key, List, unknown).
+
+pget(Key, List, Default) when is_number(Default) ->
+    case rabbit_misc:pget(Key, List) of
+        Number when is_number(Number) ->
+            Number;
+        _Other ->
+            Default
+    end;
+pget(Key, List, Default) ->
+    rabbit_misc:pget(Key, List, Default).
