@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mqtt_processor).
@@ -96,13 +96,13 @@ process_request(?CONNECT,
             _ ->
                 case creds(Username, Password, SSLLoginName) of
                     nocreds ->
-                        rabbit_log:error("MQTT login failed: no credentials provided~n"),
+                        log(error, "MQTT login failed: no credentials provided~n"),
                         {?CONNACK_CREDENTIALS, PState};
                     {invalid_creds, {undefined, Pass}} when is_list(Pass) ->
-                        rabbit_log:error("MQTT login failed: no user username is provided"),
+                        log(error, "MQTT login failed: no user username is provided"),
                         {?CONNACK_CREDENTIALS, PState};
                     {invalid_creds, {User, undefined}} when is_list(User) ->
-                        rabbit_log:error("MQTT login failed for ~p: no password provided", [User]),
+                        log(error, "MQTT login failed for ~p: no password provided", [User]),
                         {?CONNACK_CREDENTIALS, PState};
                     {UserBin, PassBin} ->
                         case process_login(UserBin, PassBin, ProtoVersion, PState) of
@@ -158,7 +158,7 @@ process_request(?PUBACK,
       true ->
         Tag = gb_trees:get(MessageId, Awaiting),
         amqp_channel:cast(Channel, #'basic.ack'{ delivery_tag = Tag }),
-        {ok, PState #proc_state{ awaiting_ack = gb_trees:delete( MessageId, Awaiting)}}
+        {ok, PState#proc_state{ awaiting_ack = gb_trees:delete(MessageId, Awaiting) }}
     end;
 
 process_request(?PUBLISH,
@@ -192,13 +192,14 @@ process_request(?PUBLISH,
 process_request(?SUBSCRIBE,
                 #mqtt_frame{
                   variable = #mqtt_frame_subscribe{
-                              message_id  = MessageId,
+                              message_id  = SubscribeMsgId,
                               topic_table = Topics},
                   payload = undefined},
                 #proc_state{channels = {Channel, _},
                             exchange = Exchange,
                             retainer_pid = RPid,
-                            send_fun = SendFun } = PState0) ->
+                            send_fun = SendFun,
+                            message_id  = StateMsgId } = PState0) ->
     check_subscribe_or_die(Topics, fun() ->
         {QosResponse, PState1} =
             lists:foldl(fun (#mqtt_topic{name = TopicName,
@@ -218,17 +219,18 @@ process_request(?SUBSCRIBE,
                        end, {[], PState0}, Topics),
         SendFun(#mqtt_frame{fixed    = #mqtt_frame_fixed{type = ?SUBACK},
                             variable = #mqtt_frame_suback{
-                                        message_id = MessageId,
+                                        message_id = SubscribeMsgId,
                                         qos_table  = QosResponse}}, PState1),
         %% we may need to send up to length(Topics) messages.
         %% if QoS is > 0 then we need to generate a message id,
         %% and increment the counter.
+        StartMsgId = safe_max_id(SubscribeMsgId, StateMsgId),
         N = lists:foldl(fun (Topic, Acc) ->
                           case maybe_send_retained_message(RPid, Topic, Acc, PState1) of
                             {true, X} -> Acc + X;
                             false     -> Acc
                           end
-                        end, MessageId, Topics),
+                        end, StartMsgId, Topics),
         {ok, PState1#proc_state{message_id = N}}
     end, PState0);
 
@@ -274,8 +276,6 @@ process_request(?PINGREQ, #mqtt_frame{}, #proc_state{ send_fun = SendFun } = PSt
 process_request(?DISCONNECT, #mqtt_frame{}, PState) ->
     {stop, PState}.
 
-%%----------------------------------------------------------------------------
-
 hand_off_to_retainer(RetainerPid, Topic, #mqtt_msg{payload = <<"">>}) ->
   rabbit_mqtt_retainer:clear(RetainerPid, Topic),
   ok;
@@ -350,11 +350,10 @@ amqp_callback({#'basic.deliver'{ consumer_tag = ConsumerTag,
                   {?QOS_0, ?QOS_0} ->
                       {ok, PState};
                   {?QOS_1, ?QOS_1} ->
-                      {ok,
-                       next_msg_id(
-                         PState #proc_state{
-                           awaiting_ack =
-                             gb_trees:insert(MsgId, DeliveryTag, Awaiting)})};
+                      Awaiting1 = gb_trees:insert(MsgId, DeliveryTag, Awaiting),
+                      PState1 = PState#proc_state{ awaiting_ack = Awaiting1 },
+                      PState2 = next_msg_id(PState1),
+                      {ok, PState2};
                   {?QOS_0, ?QOS_1} ->
                       amqp_channel:cast(
                         Channel, #'basic.ack'{ delivery_tag = DeliveryTag }),
@@ -395,10 +394,17 @@ delivery_dup({#'basic.deliver'{ redelivered = Redelivered },
         {bool, Dup} -> Redelivered orelse Dup
     end.
 
-next_msg_id(PState = #proc_state{ message_id = 16#ffff }) ->
-    PState #proc_state{ message_id = 1 };
-next_msg_id(PState = #proc_state{ message_id = MsgId }) ->
-    PState #proc_state{ message_id = MsgId + 1 }.
+ensure_valid_mqtt_message_id(Id) when Id >= 16#ffff ->
+    1;
+ensure_valid_mqtt_message_id(Id) ->
+    Id.
+
+safe_max_id(Id0, Id1) ->
+    ensure_valid_mqtt_message_id(erlang:max(Id0, Id1)).
+
+next_msg_id(PState = #proc_state{ message_id = MsgId0 }) ->
+    MsgId1 = ensure_valid_mqtt_message_id(MsgId0 + 1),
+    PState#proc_state{ message_id = MsgId1 }.
 
 %% decide at which qos level to deliver based on subscription
 %% and the message publish qos level. non-MQTT publishes are
@@ -438,8 +444,6 @@ session_present(Channel, ClientId)  ->
         _                     -> false
     end.
 
-%%----------------------------------------------------------------------------
-
 make_will_msg(#mqtt_frame_connect{ will_flag   = false }) ->
     undefined;
 make_will_msg(#mqtt_frame_connect{ will_retain = Retain,
@@ -459,7 +463,7 @@ process_login(UserBin, PassBin, ProtoVersion,
                            ssl_login_name = SslLoginName}) ->
     {ok, {_, _, _, ToPort}} = rabbit_net:socket_ends(Sock, inbound),
     {VHostPickedUsing, {VHost, UsernameBin}} = get_vhost(UserBin, SslLoginName, ToPort),
-    rabbit_log:info(
+    log(info,
         "MQTT vhost picked using ~s~n",
         [human_readable_vhost_lookup_strategy(VHostPickedUsing)]),
     case rabbit_vhost:exists(VHost) of
@@ -480,30 +484,30 @@ process_login(UserBin, PassBin, ProtoVersion,
                                     vhost = VHost}};
                         not_allowed ->
                             amqp_connection:close(Connection),
-                            rabbit_log:warning(
+                            log(warning,
                                 "MQTT login failed for ~p access_refused "
                                 "(access must be from localhost)~n",
                                 [binary_to_list(UsernameBin)]),
                             ?CONNACK_AUTH
                     end;
                 {error, {auth_failure, Explanation}} ->
-                    rabbit_log:error("MQTT login failed for ~p auth_failure: ~s~n",
+                    log(error, "MQTT login failed for ~p auth_failure: ~s~n",
                         [binary_to_list(UserBin), Explanation]),
                     ?CONNACK_CREDENTIALS;
                 {error, access_refused} ->
-                    rabbit_log:warning("MQTT login failed for ~p access_refused "
+                    log(warning, "MQTT login failed for ~p access_refused "
                     "(vhost access not allowed)~n",
                         [binary_to_list(UserBin)]),
                     ?CONNACK_AUTH;
                 {error, not_allowed} ->
                     %% when vhost allowed for TLS connection
-                    rabbit_log:warning("MQTT login failed for ~p access_refused "
+                    log(warning, "MQTT login failed for ~p access_refused "
                     "(vhost access not allowed)~n",
                         [binary_to_list(UserBin)]),
                     ?CONNACK_AUTH
             end;
         false ->
-            rabbit_log:error("MQTT login failed for ~p auth_failure: vhost ~s does not exist~n",
+            log(error, "MQTT login failed for ~p auth_failure: vhost ~s does not exist~n",
                 [binary_to_list(UserBin), VHost]),
             ?CONNACK_CREDENTIALS
     end.
@@ -700,13 +704,24 @@ ensure_queue(Qos, #proc_state{ channels      = {Channel, _},
 send_will(PState = #proc_state{will_msg = undefined}) ->
     PState;
 
-send_will(PState = #proc_state{will_msg = WillMsg = #mqtt_msg{retain = Retain, topic = Topic}, retainer_pid = RPid}) ->
+send_will(PState = #proc_state{will_msg = WillMsg = #mqtt_msg{retain = Retain,
+                                                              topic = Topic},
+                               retainer_pid = RPid,
+                               channels = {ChQos0, ChQos1}}) ->
     amqp_pub(WillMsg, PState),
     case Retain of
         false -> ok;
         true  -> hand_off_to_retainer(RPid, Topic, WillMsg)
     end,
-    PState.
+    case ChQos1 of
+        undefined -> ok;
+        _         -> amqp_channel:close(ChQos1)
+    end,
+    case ChQos0 of
+        undefined -> ok;
+        _         -> amqp_channel:close(ChQos0)
+    end,
+    PState #proc_state{ channels = {undefined, undefined} }.
 
 amqp_pub(undefined, PState) ->
     PState;
@@ -765,7 +780,6 @@ human_readable_mqtt_version(_) ->
     "N/A".
 
 send_client(Frame, #proc_state{ socket = Sock }) ->
-    %rabbit_log:info("MQTT sending frame ~p ~n", [Frame]),
     rabbit_net:port_command(Sock, rabbit_mqtt_frame:serialise(Frame)).
 
 close_connection(PState = #proc_state{ connection = undefined }) ->
@@ -809,3 +823,7 @@ check_topic_access(TopicName, Access,
                        kind = topic,
                        name = TopicName},
   rabbit_access_control:check_resource_access(User, Resource, Access).
+
+log(Level, Fmt) -> log(Level, Fmt, []).
+
+log(Level, Fmt, Args) -> rabbit_log:log(connection, Level, Fmt, Args).
