@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mqtt_reader).
@@ -63,7 +63,7 @@ info(Pid, InfoItems) ->
 
 init([KeepaliveSup, Ref, Sock]) ->
     process_flag(trap_exit, true),
-    rabbit_net:accept_ack(Ref, Sock),
+    rabbit_networking:accept_ack(Ref, Sock),
     case rabbit_net:connection_string(Sock, inbound) of
         {ok, ConnStr} ->
             log(debug, "MQTT accepting TCP connection ~p (~s)~n", [self(), ConnStr]),
@@ -372,11 +372,13 @@ maybe_process_deferred_recv(State = #state{ deferred_recv = Data, socket = Sock
     handle_info({inet_async, Sock, noref, {ok, Data}},
                 State#state{ deferred_recv = undefined }).
 
+maybe_emit_stats(undefined) ->
+    ok;
 maybe_emit_stats(State) ->
     rabbit_event:if_enabled(State, #state.stats_timer,
                             fun() -> emit_stats(State) end).
 
-emit_stats(State=#state{connection = undefined}) ->
+emit_stats(State=#state{connection = C}) when C == none; C == undefined ->
     %% Avoid emitting stats on terminate when the connection has not yet been
     %% established, as this causes orphan entries on the stats database
     State1 = rabbit_event:reset_stats_timer(State, #state.stats_timer),
@@ -403,8 +405,8 @@ info_internal(SockStat, #state{socket = Sock}) when SockStat =:= recv_oct;
                                                     SockStat =:= send_cnt;
                                                     SockStat =:= send_pend ->
     case rabbit_net:getstat(Sock, [SockStat]) of
-        {ok, [{_, I}]} -> I;
-        {error, _} -> ''
+        {ok, [{_, N}]} when is_number(N) -> N;
+        _ -> 0
     end;
 info_internal(state, State) -> info_internal(connection_state, State);
 info_internal(garbage_collection, _State) ->
