@@ -30,18 +30,23 @@
 
 #include "mongo/db/matcher/expression_leaf.h"
 
+#include <cmath>
 #include <pcrecpp.h>
 
-#include "mongo/bson/bsonobjiterator.h"
-#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonelement_comparator.h"
 #include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/config.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/matcher/path.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/stdx/memory.h"
+#include "mongo/util/mongoutils/str.h"
 
 namespace mongo {
 
-Status LeafMatchExpression::initPath(const StringData& path) {
+Status LeafMatchExpression::setPath(StringData path) {
     _path = path;
     return _elementPath.init(_path);
 }
@@ -69,11 +74,17 @@ bool ComparisonMatchExpression::equivalent(const MatchExpression* other) const {
     const ComparisonMatchExpression* realOther =
         static_cast<const ComparisonMatchExpression*>(other);
 
-    return path() == realOther->path() && _rhs.valuesEqual(realOther->_rhs);
+    if (!CollatorInterface::collatorsMatch(_collator, realOther->_collator)) {
+        return false;
+    }
+
+    const StringData::ComparatorInterface* stringComparator = nullptr;
+    BSONElementComparator eltCmp(BSONElementComparator::FieldNamesMode::kIgnore, stringComparator);
+    return path() == realOther->path() && eltCmp.evaluate(_rhs == realOther->_rhs);
 }
 
 
-Status ComparisonMatchExpression::init(const StringData& path, const BSONElement& rhs) {
+Status ComparisonMatchExpression::init(StringData path, const BSONElement& rhs) {
     _rhs = rhs;
 
     if (rhs.eoo()) {
@@ -95,13 +106,11 @@ Status ComparisonMatchExpression::init(const StringData& path, const BSONElement
             return Status(ErrorCodes::BadValue, "bad match type for ComparisonMatchExpression");
     }
 
-    return initPath(path);
+    return setPath(path);
 }
 
 
 bool ComparisonMatchExpression::matchesSingleElement(const BSONElement& e) const {
-    // log() << "\t ComparisonMatchExpression e: " << e << " _rhs: " << _rhs << "\n"
-    //<< toString() << std::endl;
 
     if (e.canonicalType() != _rhs.canonicalType()) {
         // some special cases
@@ -119,8 +128,8 @@ bool ComparisonMatchExpression::matchesSingleElement(const BSONElement& e) const
 
     // Special case handling for NaN. NaN is equal to NaN but
     // otherwise always compares to false.
-    if (isNaN(e.numberDouble()) || isNaN(_rhs.numberDouble())) {
-        bool bothNaN = isNaN(e.numberDouble()) && isNaN(_rhs.numberDouble());
+    if (std::isnan(e.numberDouble()) || std::isnan(_rhs.numberDouble())) {
+        bool bothNaN = std::isnan(e.numberDouble()) && std::isnan(_rhs.numberDouble());
         switch (matchType()) {
             case LT:
                 return false;
@@ -139,9 +148,7 @@ bool ComparisonMatchExpression::matchesSingleElement(const BSONElement& e) const
         }
     }
 
-    int x = compareElementValues(e, _rhs);
-
-    // log() << "\t\t" << x << endl;
+    int x = compareElementValues(e, _rhs, _collator);
 
     switch (matchType()) {
         case LT:
@@ -181,8 +188,7 @@ void ComparisonMatchExpression::debugString(StringBuilder& debug, int level) con
             debug << "$gte";
             break;
         default:
-            debug << " UNKNOWN - should be impossible";
-            break;
+            invariant(false);
     }
     debug << " " << _rhs.toString(false);
 
@@ -195,8 +201,8 @@ void ComparisonMatchExpression::debugString(StringBuilder& debug, int level) con
     debug << "\n";
 }
 
-void ComparisonMatchExpression::toBSON(BSONObjBuilder* out) const {
-    string opString = "";
+void ComparisonMatchExpression::serialize(BSONObjBuilder* out) const {
+    std::string opString = "";
     switch (matchType()) {
         case LT:
             opString = "$lt";
@@ -214,8 +220,7 @@ void ComparisonMatchExpression::toBSON(BSONObjBuilder* out) const {
             opString = "$gte";
             break;
         default:
-            opString = " UNKNOWN - should be impossible";
-            break;
+            invariant(false);
     }
 
     out->append(path(), BSON(opString << _rhs));
@@ -255,16 +260,14 @@ bool RegexMatchExpression::equivalent(const MatchExpression* other) const {
 }
 
 
-Status RegexMatchExpression::init(const StringData& path, const BSONElement& e) {
+Status RegexMatchExpression::init(StringData path, const BSONElement& e) {
     if (e.type() != RegEx)
         return Status(ErrorCodes::BadValue, "regex not a regex");
     return init(path, e.regex(), e.regexFlags());
 }
 
 
-Status RegexMatchExpression::init(const StringData& path,
-                                  const StringData& regex,
-                                  const StringData& options) {
+Status RegexMatchExpression::init(StringData path, StringData regex, StringData options) {
     if (regex.size() > MaxPatternSize) {
         return Status(ErrorCodes::BadValue, "Regular expression is too long");
     }
@@ -283,20 +286,19 @@ Status RegexMatchExpression::init(const StringData& path,
     _flags = options.toString();
     _re.reset(new pcrecpp::RE(_regex.c_str(), flags2options(_flags.c_str())));
 
-    return initPath(path);
+    return setPath(path);
 }
 
 bool RegexMatchExpression::matchesSingleElement(const BSONElement& e) const {
-    // log() << "RegexMatchExpression::matchesSingleElement _regex: " << _regex << " e: " << e <<
-    // std::endl;
     switch (e.type()) {
         case String:
-        case Symbol:
-            // TODO
-            // if (rm._prefix.empty())
-            return _re->PartialMatch(e.valuestr());
-        // else
-        // return !strncmp(e.valuestr(), rm._prefix.c_str(), rm._prefix.size());
+        case Symbol: {
+            // String values stored in documents can contain embedded NUL bytes. We construct a
+            // pcrecpp::StringPiece instance using the full length of the string to avoid truncating
+            // 'data' early.
+            pcrecpp::StringPiece data(e.valuestr(), e.valuestrsize() - 1);
+            return _re->PartialMatch(data);
+        }
         case RegEx:
             return _regex == e.regex() && _flags == e.regexFlags();
         default:
@@ -316,7 +318,18 @@ void RegexMatchExpression::debugString(StringBuilder& debug, int level) const {
     debug << "\n";
 }
 
-void RegexMatchExpression::toBSON(BSONObjBuilder* out) const {
+void RegexMatchExpression::serialize(BSONObjBuilder* out) const {
+    BSONObjBuilder regexBuilder(out->subobjStart(path()));
+    regexBuilder.append("$regex", _regex);
+
+    if (!_flags.empty()) {
+        regexBuilder.append("$options", _flags);
+    }
+
+    regexBuilder.doneFast();
+}
+
+void RegexMatchExpression::serializeToBSONTypeRegex(BSONObjBuilder* out) const {
     out->appendRegex(path(), _regex, _flags);
 }
 
@@ -326,12 +339,12 @@ void RegexMatchExpression::shortDebugString(StringBuilder& debug) const {
 
 // ---------
 
-Status ModMatchExpression::init(const StringData& path, int divisor, int remainder) {
+Status ModMatchExpression::init(StringData path, int divisor, int remainder) {
     if (divisor == 0)
         return Status(ErrorCodes::BadValue, "divisor cannot be 0");
     _divisor = divisor;
     _remainder = remainder;
-    return initPath(path);
+    return setPath(path);
 }
 
 bool ModMatchExpression::matchesSingleElement(const BSONElement& e) const {
@@ -351,7 +364,7 @@ void ModMatchExpression::debugString(StringBuilder& debug, int level) const {
     debug << "\n";
 }
 
-void ModMatchExpression::toBSON(BSONObjBuilder* out) const {
+void ModMatchExpression::serialize(BSONObjBuilder* out) const {
     out->append(path(), BSON("$mod" << BSON_ARRAY(_divisor << _remainder)));
 }
 
@@ -367,8 +380,8 @@ bool ModMatchExpression::equivalent(const MatchExpression* other) const {
 
 // ------------------
 
-Status ExistsMatchExpression::init(const StringData& path) {
-    return initPath(path);
+Status ExistsMatchExpression::init(StringData path) {
+    return setPath(path);
 }
 
 bool ExistsMatchExpression::matchesSingleElement(const BSONElement& e) const {
@@ -386,7 +399,7 @@ void ExistsMatchExpression::debugString(StringBuilder& debug, int level) const {
     debug << "\n";
 }
 
-void ExistsMatchExpression::toBSON(BSONObjBuilder* out) const {
+void ExistsMatchExpression::serialize(BSONObjBuilder* out) const {
     out->append(path(), BSON("$exists" << true));
 }
 
@@ -401,13 +414,53 @@ bool ExistsMatchExpression::equivalent(const MatchExpression* other) const {
 
 // ----
 
-Status TypeMatchExpression::init(const StringData& path, int type) {
+const std::string TypeMatchExpression::kMatchesAllNumbersAlias = "number";
+
+const stdx::unordered_map<std::string, BSONType> TypeMatchExpression::typeAliasMap = {
+    {typeName(NumberDouble), NumberDouble},
+    {typeName(String), String},
+    {typeName(Object), Object},
+    {typeName(Array), Array},
+    {typeName(BinData), BinData},
+    {typeName(Undefined), Undefined},
+    {typeName(jstOID), jstOID},
+    {typeName(Bool), Bool},
+    {typeName(Date), Date},
+    {typeName(jstNULL), jstNULL},
+    {typeName(RegEx), RegEx},
+    {typeName(DBRef), DBRef},
+    {typeName(Code), Code},
+    {typeName(Symbol), Symbol},
+    {typeName(CodeWScope), CodeWScope},
+    {typeName(NumberInt), NumberInt},
+    {typeName(bsonTimestamp), bsonTimestamp},
+    {typeName(NumberLong), NumberLong},
+    {typeName(NumberDecimal), NumberDecimal},
+    {typeName(MaxKey), MaxKey},
+    {typeName(MinKey), MinKey}};
+
+Status TypeMatchExpression::initWithBSONType(StringData path, int type) {
+    if (!isValidBSONType(type)) {
+        return Status(ErrorCodes::BadValue,
+                      str::stream() << "Invalid numerical $type code: " << type);
+    }
+
     _path = path;
-    _type = type;
+    _type = static_cast<BSONType>(type);
+    return _elementPath.init(_path);
+}
+
+Status TypeMatchExpression::initAsMatchingAllNumbers(StringData path) {
+    _path = path;
+    _matchesAllNumbers = true;
     return _elementPath.init(_path);
 }
 
 bool TypeMatchExpression::matchesSingleElement(const BSONElement& e) const {
+    if (_matchesAllNumbers) {
+        return e.isNumber();
+    }
+
     return e.type() == _type;
 }
 
@@ -445,7 +498,13 @@ bool TypeMatchExpression::matches(const MatchableDocument* doc, MatchDetails* de
 
 void TypeMatchExpression::debugString(StringBuilder& debug, int level) const {
     _debugAddSpace(debug, level);
-    debug << _path << " type: " << _type;
+    debug << _path << " type: ";
+    if (matchesAllNumbers()) {
+        debug << kMatchesAllNumbersAlias;
+    } else {
+        debug << _type;
+    }
+
     MatchExpression::TagData* td = getTag();
     if (NULL != td) {
         debug << " ";
@@ -454,8 +513,12 @@ void TypeMatchExpression::debugString(StringBuilder& debug, int level) const {
     debug << "\n";
 }
 
-void TypeMatchExpression::toBSON(BSONObjBuilder* out) const {
-    out->append(path(), BSON("$type" << _type));
+void TypeMatchExpression::serialize(BSONObjBuilder* out) const {
+    if (matchesAllNumbers()) {
+        out->append(path(), BSON("$type" << kMatchesAllNumbersAlias));
+    } else {
+        out->append(path(), BSON("$type" << _type));
+    }
 }
 
 bool TypeMatchExpression::equivalent(const MatchExpression* other) const {
@@ -463,136 +526,71 @@ bool TypeMatchExpression::equivalent(const MatchExpression* other) const {
         return false;
 
     const TypeMatchExpression* realOther = static_cast<const TypeMatchExpression*>(other);
-    return _path == realOther->_path && _type == realOther->_type;
-}
 
-
-// --------
-
-ArrayFilterEntries::ArrayFilterEntries() {
-    _hasNull = false;
-    _hasEmptyArray = false;
-}
-
-ArrayFilterEntries::~ArrayFilterEntries() {
-    for (unsigned i = 0; i < _regexes.size(); i++)
-        delete _regexes[i];
-    _regexes.clear();
-}
-
-Status ArrayFilterEntries::addEquality(const BSONElement& e) {
-    if (e.type() == RegEx)
-        return Status(ErrorCodes::BadValue, "ArrayFilterEntries equality cannot be a regex");
-
-    if (e.type() == Undefined) {
-        return Status(ErrorCodes::BadValue, "ArrayFilterEntries equality cannot be undefined");
-    }
-
-    if (e.type() == jstNULL) {
-        _hasNull = true;
-    }
-
-    if (e.type() == Array && e.Obj().isEmpty())
-        _hasEmptyArray = true;
-
-    _equalities.insert(e);
-    return Status::OK();
-}
-
-Status ArrayFilterEntries::addRegex(RegexMatchExpression* expr) {
-    _regexes.push_back(expr);
-    return Status::OK();
-}
-
-bool ArrayFilterEntries::equivalent(const ArrayFilterEntries& other) const {
-    if (_hasNull != other._hasNull)
+    if (_path != realOther->_path) {
         return false;
+    }
 
-    if (_regexes.size() != other._regexes.size())
-        return false;
-    for (unsigned i = 0; i < _regexes.size(); i++)
-        if (!_regexes[i]->equivalent(other._regexes[i]))
-            return false;
+    if (_matchesAllNumbers) {
+        return realOther->_matchesAllNumbers;
+    }
 
-    return _equalities == other._equalities;
+    return _type == realOther->_type;
 }
 
-void ArrayFilterEntries::copyTo(ArrayFilterEntries& toFillIn) const {
-    toFillIn._hasNull = _hasNull;
-    toFillIn._hasEmptyArray = _hasEmptyArray;
-    toFillIn._equalities = _equalities;
-    for (unsigned i = 0; i < _regexes.size(); i++)
-        toFillIn._regexes.push_back(
-            static_cast<RegexMatchExpression*>(_regexes[i]->shallowClone()));
-}
-
-void ArrayFilterEntries::debugString(StringBuilder& debug) const {
-    debug << "[ ";
-    for (BSONElementSet::const_iterator it = _equalities.begin(); it != _equalities.end(); ++it) {
-        debug << it->toString(false) << " ";
-    }
-    for (size_t i = 0; i < _regexes.size(); ++i) {
-        _regexes[i]->shortDebugString(debug);
-        debug << " ";
-    }
-    debug << "]";
-}
-
-void ArrayFilterEntries::toBSON(BSONArrayBuilder* out) const {
-    for (BSONElementSet::const_iterator it = _equalities.begin(); it != _equalities.end(); ++it) {
-        out->append(*it);
-    }
-    for (size_t i = 0; i < _regexes.size(); ++i) {
-        BSONObjBuilder regexBob;
-        _regexes[i]->toBSON(&regexBob);
-        out->append(regexBob.obj().firstElement());
-    }
-    out->doneFast();
-}
 
 // -----------
 
-Status InMatchExpression::init(const StringData& path) {
-    return initPath(path);
+Status InMatchExpression::init(StringData path) {
+    return setPath(path);
 }
 
-bool InMatchExpression::_matchesRealElement(const BSONElement& e) const {
-    if (_arrayEntries.contains(e))
-        return true;
-
-    for (unsigned i = 0; i < _arrayEntries.numRegexes(); i++) {
-        if (_arrayEntries.regex(i)->matchesSingleElement(e))
-            return true;
+std::unique_ptr<MatchExpression> InMatchExpression::shallowClone() const {
+    auto next = stdx::make_unique<InMatchExpression>();
+    next->init(path());
+    next->setCollator(_collator);
+    if (getTag()) {
+        next->setTag(getTag()->clone());
     }
-
-    return false;
+    next->_hasNull = _hasNull;
+    next->_hasEmptyArray = _hasEmptyArray;
+    next->_equalitySet = _equalitySet;
+    next->_originalEqualityVector = _originalEqualityVector;
+    for (auto&& regex : _regexes) {
+        std::unique_ptr<RegexMatchExpression> clonedRegex(
+            static_cast<RegexMatchExpression*>(regex->shallowClone().release()));
+        next->_regexes.push_back(std::move(clonedRegex));
+    }
+    return std::move(next);
 }
 
 bool InMatchExpression::matchesSingleElement(const BSONElement& e) const {
-    if (_arrayEntries.hasNull() && e.eoo())
+    if (_hasNull && e.eoo()) {
         return true;
-
-    if (_matchesRealElement(e))
+    }
+    if (_equalitySet.find(e) != _equalitySet.end()) {
         return true;
-
-    /*
-    if ( e.type() == Array ) {
-        BSONObjIterator i( e.Obj() );
-        while ( i.more() ) {
-            BSONElement sub = i.next();
-            if ( _matchesRealElement( sub ) )
-                return true;
+    }
+    for (auto&& regex : _regexes) {
+        if (regex->matchesSingleElement(e)) {
+            return true;
         }
     }
-    */
-
     return false;
 }
 
 void InMatchExpression::debugString(StringBuilder& debug, int level) const {
     _debugAddSpace(debug, level);
     debug << path() << " $in ";
-    _arrayEntries.debugString(debug);
+    debug << "[ ";
+    for (auto&& equality : _equalitySet) {
+        debug << equality.toString(false) << " ";
+    }
+    for (auto&& regex : _regexes) {
+        regex->shortDebugString(debug);
+        debug << " ";
+    }
+    debug << "]";
     MatchExpression::TagData* td = getTag();
     if (NULL != td) {
         debug << " ";
@@ -601,31 +599,333 @@ void InMatchExpression::debugString(StringBuilder& debug, int level) const {
     debug << "\n";
 }
 
-void InMatchExpression::toBSON(BSONObjBuilder* out) const {
+void InMatchExpression::serialize(BSONObjBuilder* out) const {
     BSONObjBuilder inBob(out->subobjStart(path()));
     BSONArrayBuilder arrBob(inBob.subarrayStart("$in"));
-    _arrayEntries.toBSON(&arrBob);
+    for (auto&& _equality : _equalitySet) {
+        arrBob.append(_equality);
+    }
+    for (auto&& _regex : _regexes) {
+        BSONObjBuilder regexBob;
+        _regex->serializeToBSONTypeRegex(&regexBob);
+        arrBob.append(regexBob.obj().firstElement());
+    }
+    arrBob.doneFast();
     inBob.doneFast();
 }
 
 bool InMatchExpression::equivalent(const MatchExpression* other) const {
-    if (matchType() != other->matchType())
+    if (matchType() != other->matchType()) {
         return false;
-    const InMatchExpression* realOther = static_cast<const InMatchExpression*>(other);
-    return path() == realOther->path() && _arrayEntries.equivalent(realOther->_arrayEntries);
-}
-
-LeafMatchExpression* InMatchExpression::shallowClone() const {
-    InMatchExpression* next = new InMatchExpression();
-    copyTo(next);
-    if (getTag()) {
-        next->setTag(getTag()->clone());
     }
-    return next;
+    const InMatchExpression* realOther = static_cast<const InMatchExpression*>(other);
+    if (path() != realOther->path()) {
+        return false;
+    }
+    if (_hasNull != realOther->_hasNull) {
+        return false;
+    }
+    if (_regexes.size() != realOther->_regexes.size()) {
+        return false;
+    }
+    for (size_t i = 0; i < _regexes.size(); ++i) {
+        if (!_regexes[i]->equivalent(realOther->_regexes[i].get())) {
+            return false;
+        }
+    }
+    if (!CollatorInterface::collatorsMatch(_collator, realOther->_collator)) {
+        return false;
+    }
+    // We use an element-wise comparison to check equivalence of '_equalitySet'.  Unfortunately, we
+    // can't use BSONElementSet::operator==(), as it does not use the comparator object the set is
+    // initialized with (and as such, it is not collation-aware).
+    if (_equalitySet.size() != realOther->_equalitySet.size()) {
+        return false;
+    }
+    auto thisEqIt = _equalitySet.begin();
+    auto otherEqIt = realOther->_equalitySet.begin();
+    for (; thisEqIt != _equalitySet.end(); ++thisEqIt, ++otherEqIt) {
+        const bool considerFieldName = false;
+        if (thisEqIt->woCompare(*otherEqIt, considerFieldName, _collator)) {
+            return false;
+        }
+    }
+    invariant(otherEqIt == realOther->_equalitySet.end());
+    return true;
 }
 
-void InMatchExpression::copyTo(InMatchExpression* toFillIn) const {
-    toFillIn->init(path());
-    _arrayEntries.copyTo(toFillIn->_arrayEntries);
+void InMatchExpression::_doSetCollator(const CollatorInterface* collator) {
+    _collator = collator;
+    _eltCmp = BSONElementComparator(BSONElementComparator::FieldNamesMode::kIgnore, _collator);
+
+    // We need to re-compute '_equalitySet', since our set comparator has changed.
+    _equalitySet = _eltCmp.makeBSONEltFlatSet(_originalEqualityVector);
+}
+
+Status InMatchExpression::setEqualities(std::vector<BSONElement> equalities) {
+    for (auto&& equality : equalities) {
+        if (equality.type() == BSONType::RegEx) {
+            return Status(ErrorCodes::BadValue, "InMatchExpression equality cannot be a regex");
+        }
+        if (equality.type() == BSONType::Undefined) {
+            return Status(ErrorCodes::BadValue, "InMatchExpression equality cannot be undefined");
+        }
+
+        if (equality.type() == BSONType::jstNULL) {
+            _hasNull = true;
+        } else if (equality.type() == BSONType::Array && equality.Obj().isEmpty()) {
+            _hasEmptyArray = true;
+        }
+    }
+    _originalEqualityVector = std::move(equalities);
+
+    _equalitySet = _eltCmp.makeBSONEltFlatSet(_originalEqualityVector);
+
+    return Status::OK();
+}
+
+Status InMatchExpression::addRegex(std::unique_ptr<RegexMatchExpression> expr) {
+    _regexes.push_back(std::move(expr));
+    return Status::OK();
+}
+
+// -----------
+
+const double BitTestMatchExpression::kLongLongMaxPlusOneAsDouble =
+    scalbn(1, std::numeric_limits<long long>::digits);
+
+Status BitTestMatchExpression::init(StringData path, std::vector<uint32_t> bitPositions) {
+    _bitPositions = std::move(bitPositions);
+
+    // Process bit positions into bitmask.
+    for (auto bitPosition : _bitPositions) {
+        // Checking bits > 63 is just checking the sign bit, since we sign-extend numbers. For
+        // example, the 100th bit of -1 is considered set if and only if the 63rd bit position is
+        // set.
+        bitPosition = std::min(bitPosition, 63U);
+        _bitMask |= 1ULL << bitPosition;
+    }
+
+    return setPath(path);
+}
+
+Status BitTestMatchExpression::init(StringData path, uint64_t bitMask) {
+    _bitMask = bitMask;
+
+    // Process bitmask into bit positions.
+    for (int bit = 0; bit < 64; bit++) {
+        if (_bitMask & (1ULL << bit)) {
+            _bitPositions.push_back(bit);
+        }
+    }
+
+    return setPath(path);
+}
+
+Status BitTestMatchExpression::init(StringData path,
+                                    const char* bitMaskBinary,
+                                    uint32_t bitMaskLen) {
+    for (uint32_t byte = 0; byte < bitMaskLen; byte++) {
+        char byteAt = bitMaskBinary[byte];
+        if (!byteAt) {
+            continue;
+        }
+
+        // Build _bitMask with the first 8 bytes of the bitMaskBinary.
+        if (byte < 8) {
+            _bitMask |= static_cast<uint64_t>(byteAt) << byte * 8;
+        } else {
+            // Checking bits > 63 is just checking the sign bit, since we sign-extend numbers. For
+            // example, the 100th bit of -1 is considered set if and only if the 63rd bit position
+            // is set.
+            _bitMask |= 1ULL << 63;
+        }
+
+        for (int bit = 0; bit < 8; bit++) {
+            if (byteAt & (1 << bit)) {
+                _bitPositions.push_back(8 * byte + bit);
+            }
+        }
+    }
+
+    return setPath(path);
+}
+
+bool BitTestMatchExpression::needFurtherBitTests(bool isBitSet) const {
+    const MatchType mt = matchType();
+
+    return (isBitSet && (mt == BITS_ALL_SET || mt == BITS_ANY_CLEAR)) ||
+        (!isBitSet && (mt == BITS_ALL_CLEAR || mt == BITS_ANY_SET));
+}
+
+bool BitTestMatchExpression::performBitTest(long long eValue) const {
+    const MatchType mt = matchType();
+
+    switch (mt) {
+        case BITS_ALL_SET:
+            return (eValue & _bitMask) == _bitMask;
+        case BITS_ALL_CLEAR:
+            return (~eValue & _bitMask) == _bitMask;
+        case BITS_ANY_SET:
+            return eValue & _bitMask;
+        case BITS_ANY_CLEAR:
+            return ~eValue & _bitMask;
+        default:
+            invariant(false);
+    }
+}
+
+bool BitTestMatchExpression::performBitTest(const char* eBinary, uint32_t eBinaryLen) const {
+    const MatchType mt = matchType();
+
+    // Test each bit position.
+    for (auto bitPosition : _bitPositions) {
+        bool isBitSet;
+        if (bitPosition >= eBinaryLen * 8) {
+            // If position to test is longer than the data to test against, zero-extend.
+            isBitSet = false;
+        } else {
+            // Map to byte position and bit position within that byte. Note that byte positions
+            // start at position 0 in the char array, and bit positions start at the least
+            // significant bit.
+            int bytePosition = bitPosition / 8;
+            int bit = bitPosition % 8;
+            char byte = eBinary[bytePosition];
+
+            isBitSet = byte & (1 << bit);
+        }
+
+        if (!needFurtherBitTests(isBitSet)) {
+            // If we can skip the rest fo the tests, that means we succeeded with _ANY_ or failed
+            // with _ALL_.
+            return mt == BITS_ANY_SET || mt == BITS_ANY_CLEAR;
+        }
+    }
+
+    // If we finished all the tests, that means we succeeded with _ALL_ or failed with _ANY_.
+    return mt == BITS_ALL_SET || mt == BITS_ALL_CLEAR;
+}
+
+bool BitTestMatchExpression::matchesSingleElement(const BSONElement& e) const {
+    // Validate 'e' is a number or a BinData.
+    if (!e.isNumber() && e.type() != BSONType::BinData) {
+        return false;
+    }
+
+    if (e.type() == BSONType::BinData) {
+        int eBinaryLen;  // Length of eBinary (in bytes).
+        const char* eBinary = e.binData(eBinaryLen);
+        return performBitTest(eBinary, eBinaryLen);
+    }
+
+    invariant(e.isNumber());
+
+    if (e.type() == BSONType::NumberDouble) {
+        double eDouble = e.numberDouble();
+
+        // NaN doubles are rejected.
+        if (std::isnan(eDouble)) {
+            return false;
+        }
+
+        // Integral doubles that are too large or small to be represented as a 64-bit signed
+        // integer are treated as 0. We use 'kLongLongMaxAsDouble' because if we just did
+        // eDouble > 2^63-1, it would be compared against 2^63. eDouble=2^63 would not get caught
+        // that way.
+        if (eDouble >= kLongLongMaxPlusOneAsDouble ||
+            eDouble < std::numeric_limits<long long>::min()) {
+            return false;
+        }
+
+        // This checks if e is an integral double.
+        if (eDouble != static_cast<double>(static_cast<long long>(eDouble))) {
+            return false;
+        }
+    }
+
+    long long eValue = e.numberLong();
+    return performBitTest(eValue);
+}
+
+void BitTestMatchExpression::debugString(StringBuilder& debug, int level) const {
+    _debugAddSpace(debug, level);
+
+    debug << path() << " ";
+
+    switch (matchType()) {
+        case BITS_ALL_SET:
+            debug << "$bitsAllSet:";
+            break;
+        case BITS_ALL_CLEAR:
+            debug << "$bitsAllClear:";
+            break;
+        case BITS_ANY_SET:
+            debug << "$bitsAnySet:";
+            break;
+        case BITS_ANY_CLEAR:
+            debug << "$bitsAnyClear:";
+            break;
+        default:
+            invariant(false);
+    }
+
+    debug << " [";
+    for (size_t i = 0; i < _bitPositions.size(); i++) {
+        debug << _bitPositions[i];
+        if (i != _bitPositions.size() - 1) {
+            debug << ", ";
+        }
+    }
+    debug << "]";
+
+    MatchExpression::TagData* td = getTag();
+    if (td) {
+        debug << " ";
+        td->debugString(&debug);
+    }
+}
+
+void BitTestMatchExpression::serialize(BSONObjBuilder* out) const {
+    std::string opString = "";
+
+    switch (matchType()) {
+        case BITS_ALL_SET:
+            opString = "$bitsAllSet";
+            break;
+        case BITS_ALL_CLEAR:
+            opString = "$bitsAllClear";
+            break;
+        case BITS_ANY_SET:
+            opString = "$bitsAnySet";
+            break;
+        case BITS_ANY_CLEAR:
+            opString = "$bitsAnyClear";
+            break;
+        default:
+            invariant(false);
+    }
+
+    BSONArrayBuilder arrBob;
+    for (auto bitPosition : _bitPositions) {
+        arrBob.append(bitPosition);
+    }
+    arrBob.doneFast();
+
+    out->append(path(), BSON(opString << arrBob.arr()));
+}
+
+bool BitTestMatchExpression::equivalent(const MatchExpression* other) const {
+    if (matchType() != other->matchType()) {
+        return false;
+    }
+
+    const BitTestMatchExpression* realOther = static_cast<const BitTestMatchExpression*>(other);
+
+    std::vector<uint32_t> myBitPositions = getBitPositions();
+    std::vector<uint32_t> otherBitPositions = realOther->getBitPositions();
+    std::sort(myBitPositions.begin(), myBitPositions.end());
+    std::sort(otherBitPositions.begin(), otherBitPositions.end());
+
+    return path() == realOther->path() && myBitPositions == otherBitPositions;
 }
 }
